<?php
namespace backend\models;

use common\models\User;
use yii\base\Model;
use Yii;
/**
 * Password reset request form
 */
class PasswordResetRequestForm extends Model
{
    public $email;

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            ['email', 'filter', 'filter' => 'trim'],
            ['email', 'required'],
            ['email', 'email'],
            ['email', 'exist',
                'targetClass' => '\common\models\User',
                'filter' => ['status' => User::STATUS_ACTIVE],
                'message' => 'Please enter the fields correctly.'
            ],
        ];
    }

    /**
     * Sends an email with a link, for resetting the password.
     *
     * @return boolean whether the email was send
     */
    public function sendEmail()
    {

        $user = User::findOne([
            'status' => User::STATUS_ACTIVE,
            'email' => $this->email,
        ]);

        if ($user) {
            if (!User::isPasswordResetTokenValid($user->password_reset_token)) {
                $user->generatePasswordResetToken();
            }

            if ($user->save()) {
                return \Yii::$app->mailer->compose(['html' => 'passwordResetToken-html', 'text' => 'passwordResetToken-text'], ['user' => $user])
                    //->setFrom([\Yii::$app->params['supportEmail'] => \Yii::$app->name ])
                    ->setFrom(['noreply@'.Yii::$app->getRequest()->serverName => \Yii::$app->name])
                    ->setTo($this->email)
                    ->setSubject(\Yii::t('app','Password reset for ') . \Yii::$app->name)
                    ->send();
            }
        }

        return false;
    }
}
